#include <iostream>
#include "exception.h"
#include "Vector.h"
#include "CSLRMatrx.h"
#include <iomanip>
using namespace std;

CSLRMatrix::CSLRMatrix(double* p_adiag, double* p_altr, double* p_jptr, double* p_iptr) :
    adiag(new double[_msize(p_adiag) / sizeof(double)]),
    altr(new double[_msize(p_altr) / sizeof(double)]),
    jptr(new double[_msize(p_jptr) / sizeof(double)]),
    iptr(new double[_msize(p_iptr) / sizeof(double)])
{
    for (int i = 0; i < _msize(p_adiag) / sizeof(double); ++i)
    {
        adiag[i] = p_adiag[i];
    }
    for (int i = 0; i < _msize(p_altr) / sizeof(double); ++i)
    {
        altr[i] = p_altr[i];
        jptr[i] = p_jptr[i];
    }
    for (int i = 0; i < _msize(p_iptr) / sizeof(double); ++i)
    {
        iptr[i] = p_iptr[i];
    }
}
CSLRMatrix::CSLRMatrix(const CSLRMatrix& p_mat) :
    adiag(new double[_msize(p_mat.adiag) / sizeof(double)]),
    altr(new double[_msize(p_mat.altr) / sizeof(double)]),
    jptr(new double[_msize(p_mat.jptr) / sizeof(double)]),
    iptr(new double[_msize(p_mat.iptr) / sizeof(double)])
{
    for (int i = 0; i < _msize(p_mat.adiag) / sizeof(double); ++i)
    {
        adiag[i] = p_mat.adiag[i];
    }
    for (int i = 0; i < _msize(p_mat.altr) / sizeof(double); ++i)
    {
        altr[i] = p_mat.altr[i];
        jptr[i] = p_mat.jptr[i];
    }
    for (int i = 0; i < _msize(p_mat.iptr) / sizeof(double); ++i)
    {
        iptr[i] = p_mat.iptr[i];
    }
}




CSLRMatrix::~CSLRMatrix() {
    delete[] adiag;
    delete[] altr;
    delete[] iptr;
    delete[] jptr;
}

std::istream& operator>>(std::istream& is, CSLRMatrix& m) {

    printf("\nenter the elements that are on the main diagonal\n");
    for (int i = 0; i < _msize(m.adiag) / sizeof(double); i++)
        is >> m.adiag[i];
    printf("\nenter nonzero matrix elements\n");
    for (int i = 0; i < _msize(m.altr) / sizeof(double); i++)
        is >> m.altr[i];
    printf("\nin which columns are these elements\n");
    for (int i = 0; i < _msize(m.jptr) / sizeof(double); i++)
        is >> m.jptr[i];
    printf("\nWith what position in the array of all non - zero elements does the i-th line begin matrices\n");
    for (int i = 0; i < _msize(m.iptr) / sizeof(double); i++)
        is >> m.iptr[i];
    return is;
}
std::ostream& operator<<(std::ostream& os, const CSLRMatrix& mat) {
    for (int i = 0; i < _msize(mat.adiag) / sizeof(double); i++) {
        for (int j = 0; j < _msize(mat.adiag) / sizeof(double); j++) {
            double val = 0;
            if (i == j)
            {
                val = mat.adiag[i];
            }
            else
            {
                for (int k = mat.iptr[i] - 1; k < mat.iptr[i + 1] - 1; k++) {
                    if (mat.jptr[k] - 1 == j) {
                        val = mat.altr[k];
                        break;
                    }
                }
                for (int k = mat.iptr[j] - 1; k < mat.iptr[j + 1] - 1; k++) {
                    if (mat.jptr[k] - 1 == i) {
                        val = mat.altr[k];
                        break;
                    }
                }
            }
            os << setw(4) << val;
        }
        os << std::endl << std::endl;
    }
    return os;
}

int CSLRMatrix::matsize() const
{
    return _msize(adiag) / sizeof(double);
}
int CSLRMatrix::nzn() const
{
    int ans = _msize(altr) / sizeof(double) * 2;
    for (int i = 0; i < _msize(adiag) / sizeof(double); i++)
    {
        if (adiag[i] != 0) ans++;
    }
    return ans;
}

const CSLRMatrix& CSLRMatrix:: operator=(const CSLRMatrix& m) {
    if (this == &m) { return *this; }
    else
    {
        if (_msize(adiag) / sizeof(double) != sizeof(m.adiag) / sizeof(double))
        {
            delete[](*this).adiag;
            delete[](*this).altr;
            delete[](*this).jptr;
            delete[](*this).iptr;
            (*this).adiag = new double[_msize(m.adiag) / sizeof(double)];
            (*this).altr = new double[_msize(m.altr) / sizeof(double)];
            (*this).jptr = new double[_msize(m.jptr) / sizeof(double)];
            (*this).iptr = new double[_msize(m.iptr) / sizeof(double)];
        }
        for (int i = 0; i < _msize(m.adiag) / sizeof(double); ++i)
        {
            adiag[i] = m.adiag[i];
        }
        for (int i = 0; i < _msize(m.altr) / sizeof(double); ++i)
        {
            altr[i] = m.altr[i];
            jptr[i] = m.jptr[i];
        }
        for (int i = 0; i < _msize(m.iptr) / sizeof(double); ++i)
        {
            iptr[i] = m.iptr[i];
        }
        return *this;
    }

}

Vector operator*(const CSLRMatrix& mat, const Vector& v)
{
    if (mat.matsize() != v.dimension())
    {
        throw IncompatibleDimException{};
    }
    Vector res(v);
    for (int i = 0; i < mat.matsize(); i++)
    {
        res[i] = v[i] * mat.adiag[i];
    }
    for (int i = 0; i < mat.matsize(); i++)
    {
        for (int j = mat.iptr[i] - 1; j < mat.iptr[i + 1] - 1; j++)
        {
            res[i] += v[mat.jptr[j] - 1] * mat.altr[j];
            res[mat.jptr[j] - 1] += v[i] * mat.altr[j];
        }
    }
    return res;
}

CSLRMatrix operator*(const CSLRMatrix& m, double scalar)
{
    CSLRMatrix result(m.adiag, m.altr, m.jptr, m.iptr);
    for (int i = 0; i < _msize(result.adiag) / sizeof(double); i++)
    {
        result.adiag[i] *= scalar;
    }
    for (int i = 0; i < _msize(result.altr) / sizeof(double); i++)
    {
        result.altr[i] *= scalar;
    }
    return result;
}

CSLRMatrix operator*(double scalar, const CSLRMatrix& m)
{
    return m * scalar;
}






